unit ApeEngines;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "ApeEngines.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  ApePreferences, ApeUtils, ResourceDisplayFrame, ApeCaches, comctrls,
  UnicodeUtils, XDOM_2_4, SysUtils, Contnrs, Classes, Controls;

type
  EInuseComponentError = class(Exception);

  TApeMappingRepository = class;
  TApeUrnRepository = class;

  TApeMappingRepository = class(TComponent)
  private
    FDisplayMappingCacheManager: TApeMappingCacheManager;
    FDisplayMappingCatalog: TApeMappingCatalog;
    FBeginEvaluateEvent: TApeBeginEvaluateEvent;
    FEndEvaluateEvent: TApeEndEvaluateEvent;
    FInfoFilesCacheManager: TApeInfoCacheManager;
    FInfoFilesCatalog: TApeMappingCatalog;
    FResolveMappingCacheManager: TApeMappingCacheManager;
    FResolveMappingCatalog: TApeMappingCatalog;
    function SaveMappingCatalogToFile(const Path: TFilename;
                                      const MappingCatalog: TApeMappingCatalog): boolean;
    function UpdateMappingCatalogFromURL(const URL: wideString;
                                         const MappingCatalog: TApeMappingCatalog): boolean;
  protected
    FUserPreferences: TApePreferences;
    procedure DoBeginEvaluate(Sender: TObject;
                              FileURL: wideString); virtual;
    procedure DoEndEvaluate(Sender: TObject;
                            FileURL: wideString;
                            Success: boolean); virtual;
    function GetCacheSize: integer; virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetCacheSize(const Value: integer); virtual;
    procedure SetUserPreferences(const Value: TApePreferences); virtual;
    function UpdateLocalDisplayMappingCatalogFromURL: boolean;
    function UpdateLocalInfoFilesCatalogFromURL: boolean;
    function UpdateLocalResolveMappingCatalogFromURL: boolean;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure ClearCache; virtual;
    function AddMappingUrl(const Url: string): boolean; virtual;
    function DisplaySourceMappingCount: integer; virtual;
    function DisplaySourceMappingURN(index: integer): wideString; virtual;
    function DisplayTargetCount(sourceURN: wideString): integer; virtual;
    function DisplayTargetTitle(sourceURN: wideString; index: integer): wideString; virtual;
    function DisplayTargetURL(sourceURN: wideString; index: integer): wideString; virtual;
    procedure FindDisplayURLs(urn: wideString;
                              UrlList: TdomNameValueList); virtual;
    procedure FindResolveURNs(urn: wideString;
                              UrnList: TdomNameValueList); virtual;
    function FindTargetMappingLabel(urn: wideString): wideString; virtual;
    function FindURNLabel(urn: wideString): wideString; virtual;
    function InfoSourceMappingCount: integer; virtual;
    function InfoSourceMappingURN(index: integer): wideString; virtual;
    function InfoTargetCount(sourceURN: wideString): integer; virtual;
    function InfoTargetTitle(sourceURN: wideString; index: integer): wideString; virtual;
    function InfoTargetURL(sourceURN: wideString; index: integer): wideString; virtual;
    procedure OpenCatalogAdminDlg; virtual;
    function RequestDisplayPacket(const urnList: TdomWideStringList): TApeDisplayPacket; virtual;
    function ResolveSourceCount: integer; virtual;
    function ResolveSourceURN(index: integer): wideString; virtual;
    function ResolveTargetCount(sourceURN: wideString): integer; virtual;
    function ResolveTargetTitle(sourceURN: wideString; index: integer): wideString; virtual;
    function ResolveTargetURL(sourceURN: wideString; index: integer): wideString; virtual;
    function UpdateLocalCatalogs: boolean; virtual;
    function UpdateLocalDisplayMappingCatalog(const MappingCatalog: TApeMappingCatalog): boolean; virtual;
    function UpdateLocalInfoFilesCatalog(const MappingCatalog: TApeMappingCatalog): boolean; virtual;
    function UpdateLocalResolveMappingCatalog(const MappingCatalog: TApeMappingCatalog): boolean; virtual;
    property DisplayMappingCacheManager: TApeMappingCacheManager read FDisplayMappingCacheManager;
    property DisplayMappingCatalog: TApeMappingCatalog read FDisplayMappingCatalog;
    property InfoFilesCacheManager: TApeInfoCacheManager read FInfoFilesCacheManager;
    property InfoFilesCatalog: TApeMappingCatalog read FInfoFilesCatalog;
    property ResolveMappingCacheManager: TApeMappingCacheManager read FResolveMappingCacheManager;
    property ResolveMappingCatalog: TApeMappingCatalog read FResolveMappingCatalog;
  published
    property CacheSize: integer read GetCacheSize write SetCacheSize default -1;
    property UserPreferences: TApePreferences read FUserPreferences write SetUserPreferences;
    property OnBeginEvaluate: TApeBeginEvaluateEvent read FBeginEvaluateEvent write FBeginEvaluateEvent;
    property OnEndEvaluate: TApeEndEvaluateEvent read FEndEvaluateEvent write FEndEvaluateEvent;
  end;

  TApeUrnRepository = class(TComponent)
  protected
    FMappingRepository: TApeMappingRepository;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetMappingRepository(const Value: TApeMappingRepository); virtual;
  public
    procedure FindDisplayURLs(const urn: wideString;
                              const UrlList: TdomNameValueList); virtual;
    function FindTargetMappingLabel(const urn: wideString): wideString; virtual;
    function FindURNLabel(const urn: wideString): wideString; virtual;
    function OpenCatalogAdminDlg: boolean; virtual;
    function RequestDisplayPacket(const urnList: TdomWideStringList): TApeDisplayPacket; virtual;
    procedure ResolveAndDisplayURN(const urn: wideString;
                                   const UrlList: TdomNameValueList); virtual;
    procedure ResolveUrn(const urn: wideString;
                         const DisplayableUrnsList: TdomNameValueList); virtual;
  published
    property MappingRepository: TApeMappingRepository read FMappingRepository write SetMappingRepository;
  end;

procedure Register;

const
  MAPPING_CATALOG_SAVE_FORMAT: TdomEncodingType = etUTF_8;

implementation

uses
  ApeCatalogDlg;

procedure Register;
begin
  RegisterComponents('Ape', [ TApeUrnRepository,
                              TApeMappingRepository
                            ]);
end;

{ TApeMappingRepository }

constructor TApeMappingRepository.Create(aOwner: TComponent);
begin
  inherited;
  FDisplayMappingCatalog:= TApeMappingCatalog.Create;
  FDisplayMappingCacheManager:= TApeMappingCacheManager.create(FDisplayMappingCatalog);
  FDisplayMappingCacheManager.OnBeginEvaluate:= DoBeginEvaluate;
  FDisplayMappingCacheManager.OnEndEvaluate:= DoEndEvaluate;

  FInfoFilesCatalog:= TApeMappingCatalog.Create;
  FInfoFilesCacheManager:= TApeInfoCacheManager.create(FInfoFilesCatalog);
  FInfoFilesCacheManager.OnBeginEvaluate:= DoBeginEvaluate;
  FInfoFilesCacheManager.OnEndEvaluate:= DoEndEvaluate;

  FResolveMappingCatalog:= TApeMappingCatalog.Create;
  FResolveMappingCacheManager:= TApeMappingCacheManager.create(FResolveMappingCatalog);
  FResolveMappingCacheManager.OnBeginEvaluate:= DoBeginEvaluate;
  FResolveMappingCacheManager.OnEndEvaluate:= DoEndEvaluate;

  CacheSize:= -1;
end;

destructor TApeMappingRepository.Destroy;
begin
  FDisplayMappingCacheManager.free;
  FInfoFilesCacheManager.free;
  FResolveMappingCacheManager.free;
  FDisplayMappingCatalog.Free;
  FInfoFilesCatalog.Free;
  FResolveMappingCatalog.Free;
  inherited;
end;

procedure TApeMappingRepository.ClearCache;
begin
  FDisplayMappingCacheManager.Clear;
  FInfoFilesCacheManager.Clear;
  FResolveMappingCacheManager.Clear;
end;

function TApeMappingRepository.DisplaySourceMappingCount: integer;
begin
  result:= FDisplayMappingCatalog.SourceMappingCount;
end;

function TApeMappingRepository.DisplaySourceMappingURN(index: integer): wideString;
begin
  result:= FDisplayMappingCatalog.SourceMappingURN(index);
end;

function TApeMappingRepository.DisplayTargetCount(
  sourceURN: wideString): integer;
begin
  result:= FDisplayMappingCatalog.TargetCountByName(sourceUrn);
end;

function TApeMappingRepository.DisplayTargetTitle(sourceURN: wideString;
  index: integer): wideString;
begin
  result:= FDisplayMappingCatalog.TargetTitleByName(sourceURN,index);
end;

function TApeMappingRepository.DisplayTargetURL(sourceURN: wideString;
  index: integer): wideString;
begin
  result:= FDisplayMappingCatalog.TargetURLByName(sourceURN,index);
end;

procedure TApeMappingRepository.DoBeginEvaluate(Sender: TObject;
                                                FileURL: wideString);
begin
  if assigned(FBeginEvaluateEvent)
    then FBeginEvaluateEvent(self, FileURL);
end;

procedure TApeMappingRepository.DoEndEvaluate(Sender: TObject;
                                              FileURL: wideString;
                                              Success: boolean);
begin
  if assigned(FEndEvaluateEvent)
    then FEndEvaluateEvent(self, FileURL, Success);
end;

function TApeMappingRepository.GetCacheSize: integer;
begin
  result:= FDisplayMappingCacheManager.CacheSize;
end;

procedure TApeMappingRepository.Notification(AComponent: TComponent;
                                             Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then begin
    if AComponent = FUserPreferences then begin
      FUserPreferences:= nil;
    end;
  end;
end;

procedure TApeMappingRepository.SetCacheSize(const Value: integer);
begin
  FDisplayMappingCacheManager.CacheSize:= Value;
  FInfoFilesCacheManager.CacheSize:=      Value;
  FResolveMappingCacheManager.CacheSize:= Value;
end;

procedure TApeMappingRepository.SetUserPreferences(const Value: TApePreferences);
begin
  FUserPreferences:= Value;
end;

procedure TApeMappingRepository.FindDisplayURLs(urn: wideString;
                                                UrlList: TdomNameValueList);
begin
  FDisplayMappingCacheManager.GetTargetAndMappingBySourceURI(urn,UrlList);
end;

procedure TApeMappingRepository.FindResolveURNs(urn: wideString;
                                                UrnList: TdomNameValueList);
begin
  FResolveMappingCacheManager.GetTargetAndMappingBySourceURI(urn,UrnList);
end;

function TApeMappingRepository.FindTargetMappingLabel(
  urn: wideString): wideString;
begin
  result:= FDisplayMappingCatalog.TargetMappingTitle(urn);
end;

function TApeMappingRepository.FindURNLabel(urn: wideString): wideString;
begin
  FInfoFilesCacheManager.GetLabelByURN(urn, result);
end;

function TApeMappingRepository.InfoSourceMappingCount: integer;
begin
  result:= FInfoFilesCatalog.SourceMappingCount;
end;

function TApeMappingRepository.InfoSourceMappingURN(index: integer): wideString;
begin
  result:= FInfoFilesCatalog.SourceMappingURN(index);
end;

function TApeMappingRepository.InfoTargetCount(
  sourceURN: wideString): integer;
begin
  result:= FInfoFilesCatalog.TargetCountByName(sourceUrn);
end;

function TApeMappingRepository.InfoTargetTitle(sourceURN: wideString;
  index: integer): wideString;
begin
  result:= FInfoFilesCatalog.TargetTitleByName(sourceURN,index);
end;

function TApeMappingRepository.InfoTargetURL(sourceURN: wideString;
  index: integer): wideString;
begin
  result:= FInfoFilesCatalog.TargetURLByName(sourceURN,index);
end;

function TApeMappingRepository.AddMappingUrl(const Url: string): boolean;
var
  CatalogDlg: TCatalogDialog;
begin
  if not assigned(UserPreferences)
    then raise EInvalidOperation.Create('UserPreferences not specified.');
  if UserPreferences.ActiveUserIndex = -1
    then raise EInvalidOperation.Create('No active profile.');

  CatalogDlg:= TCatalogDialog.Create(self);
  try
    with CatalogDlg do begin
      MappingRepository:= self;
      Result := AddMappingUrl(Url);
      if Result then begin
        if DisplayMappingModified then begin
          UpdateLocalDisplayMappingCatalog(DisplayMappingCatalog);
          SaveMappingCatalogToFile(UserPreferences.ActiveDisplayMappingDoc,DisplayMappingCatalog);
        end;
        if InfoFilesModified then begin
          UpdateLocalInfoFilesCatalog(InfoFilesCatalog);
          SaveMappingCatalogToFile(UserPreferences.ActiveInfoFilesDoc,InfoFilesCatalog);
        end;
        if ResolveMappingModified then begin
          UpdateLocalResolveMappingCatalog(ResolveMappingCatalog);
          SaveMappingCatalogToFile(UserPreferences.ActiveResolveMappingDoc,ResolveMappingCatalog);
        end;
      end;
    end;
  finally
    CatalogDlg.free;
  end;
end;

procedure TApeMappingRepository.OpenCatalogAdminDlg;
var
  CatalogDlg: TCatalogDialog;
begin
  if not assigned(UserPreferences)
    then raise EInvalidOperation.Create('UserPreferences not specified.');
  if UserPreferences.ActiveUserIndex = -1
    then raise EInvalidOperation.Create('No active profile.');

  CatalogDlg:= TCatalogDialog.Create(self);
  try
    with CatalogDlg do begin
      Caption:= concat('Mapping Catalogs for Profile "',UserPreferences.Users[UserPreferences.ActiveUserIndex].ProfileName,'"');
      MappingRepository:= self;
      if ShowModal = mrOK then begin
        if DisplayMappingModified then begin
          UpdateLocalDisplayMappingCatalog(DisplayMappingCatalog);
          SaveMappingCatalogToFile(UserPreferences.ActiveDisplayMappingDoc,DisplayMappingCatalog);
        end;
        if InfoFilesModified then begin
          UpdateLocalInfoFilesCatalog(InfoFilesCatalog);
          SaveMappingCatalogToFile(UserPreferences.ActiveInfoFilesDoc,InfoFilesCatalog);
        end;
        if ResolveMappingModified then begin
          UpdateLocalResolveMappingCatalog(ResolveMappingCatalog);
          SaveMappingCatalogToFile(UserPreferences.ActiveResolveMappingDoc,ResolveMappingCatalog);
        end;
      end;
    end;
  finally
    CatalogDlg.free;
  end;
end;

function TApeMappingRepository.RequestDisplayPacket(const urnList: TdomWideStringList): TApeDisplayPacket;
var
  i,j,k: integer;
  DisplayableUrnsList: TdomNameValueList;
  DisplayUrlsList: TdomNameValueList;
  displayLabel: wideString;
  mappingLabel: wideString;
  resourceLabel: wideString;
begin
  result:= TApeDisplayPacket.create;

  DisplayableUrnsList:= TdomNameValueList.create;
  DisplayUrlsList:= TdomNameValueList.create;
  try

    for i:= 0 to pred(urnList.count) do begin
      if urnList[i] <> '' then begin
        DisplayableUrnsList.clear;
        FResolveMappingCacheManager.GetTargetAndMappingBySourceURI(urnList[i],DisplayableUrnsList);
        for j:= 0 to pred(DisplayableUrnsList.length) do begin

          mappingLabel:= FInfoFilesCatalog.TargetTitleByName(DisplayableUrnsList.values[j],0);
          result.appendMappingLabel(DisplayableUrnsList.values[j],mappingLabel,'');

          result.appendDisplay(DisplayableUrnsList.values[j],DisplayableUrnsList.names[j]);

          if FInfoFilesCacheManager.GetLabelByURN(DisplayableUrnsList.names[j],displayLabel)
           then result.appendDisplayLabel(DisplayableUrnsList.values[j],DisplayableUrnsList.names[j],displayLabel,'');

          DisplayUrlsList.clear;
          FDisplayMappingCacheManager.GetTargetAndMappingBySourceURI(DisplayableUrnsList.names[j],DisplayUrlsList);
          for k:= 0 to pred(DisplayUrlsList.length) do begin

            result.appendResource(DisplayableUrnsList.values[j],
                                  DisplayableUrnsList.names[j],
                                  DisplayUrlsList.names[k],
                                  DisplayUrlsList.values[k]);

            resourceLabel:= FDisplayMappingCatalog.TargetMappingTitle(DisplayUrlsList.values[k]);
            result.appendResourceLabel(DisplayableUrnsList.values[j],DisplayableUrnsList.names[j],DisplayUrlsList.names[k],resourceLabel,'');
          end;
        end;
      end;
    end;

  finally
    DisplayableUrnsList.free;
    DisplayUrlsList.free;
  end;
end;

function TApeMappingRepository.ResolveSourceCount: integer;
begin
  result:= FResolveMappingCatalog.SourceMappingCount;
end;

function TApeMappingRepository.ResolveSourceURN(index: integer): wideString;
begin
  result:= FResolveMappingCatalog.SourceMappingURN(index);
end;

function TApeMappingRepository.ResolveTargetCount(
  sourceURN: wideString): integer;
begin
  result:= FResolveMappingCatalog.TargetCountByName(sourceUrn);
end;

function TApeMappingRepository.ResolveTargetTitle(sourceURN: wideString;
  index: integer): wideString;
begin
  result:= FResolveMappingCatalog.TargetTitleByName(sourceURN,index);
end;

function TApeMappingRepository.ResolveTargetURL(sourceURN: wideString;
  index: integer): wideString;
begin
  result:= FResolveMappingCatalog.TargetURLByName(sourceURN,index);
end;

function TApeMappingRepository.SaveMappingCatalogToFile(const Path: TFilename;
                                                        const MappingCatalog: TApeMappingCatalog): boolean;
var
  fFileOut: TFileStream;
  directoryname: string;
  outputStr: string;
  outputWideStr: wideString;
begin
  directoryname:= ExtractFilePath(path);
  if not DirectoryExists(directoryname)
    then if not ForceDirectories(directoryname)
      then raise Exception.Create('Cannot create '+ directoryname);
  fFileOut:= TFileStream.Create(Path,fmCreate);
  case MAPPING_CATALOG_SAVE_FORMAT of
    etUTF_8: begin
      outputStr:= MappingCatalog.GetAsUTF8Str;
      fFileOut.Write(outputStr[1],Length(outputStr));
      fFileOut.Free;
      result:= true;
    end;
    etUTF_16BE: begin
      outputWideStr:= MappingCatalog.GetAsUTF16BEStr;
      fFileOut.Write(outputWideStr[1],Length(outputWideStr)*2);
      fFileOut.Free;
      result:= true;
    end;
    else result:= false;
  end;
end;

function TApeMappingRepository.UpdateMappingCatalogFromURL(const URL: wideString;
                                                           const MappingCatalog: TApeMappingCatalog): boolean;
// Returns 'true', if the mapping base has been successfully analyzed.
// Returns 'false' otherwise.
var
  dummyDoc,assocBaseDoc: TdomDocument;
  filename: TFilename;
  server,query,fragment: string;
  domImpl: TDomImplementation;
  docBuilder: TXmlDocBuilder;
  standardDomReader: TXmlStandardDomReader;
  xmlToDomParser: TXmlToDomParser;
begin

  if not UriWideStrToFilename(url,filename,server,query,fragment)
    then begin result:= false; exit; end;
  if not fileExists(fileName)
    then begin result:= false; exit; end;

  domImpl:= TDomImplementation.create(nil);
  docBuilder:= TXmlDocBuilder.create(nil);
  StandardDomReader:= TXmlStandardDomReader.create(nil);
  xmlToDomParser:= TXmlToDomParser.create(nil);
  xmlToDomParser.DomImpl:= domImpl;
  with standardDomReader do begin
    PrefixMapping:= true;
    SuppressXmlns:= true;
    NextHandler:= docBuilder;
  end;
  docBuilder.BuildNamespaceTree:= true;

  try
    try

      with xmlToDomParser do begin
        dummyDoc:= FileToDom(Filename);
        try
          result:= dummyDoc.prepareASModels;
          if result
            then result:= dummyDoc.validate(erReplace);
          if result then begin
            assocBaseDoc:= DOMImpl.createDocument('dummy',nil);
            try
              assocBaseDoc.clear;
              assocBaseDoc.setBaseUri(url);
              docBuilder.referenceNode:= assocBaseDoc;
              result:= standardDomReader.parse(dummyDoc);
              if result
                then MappingCatalog.UpdateFromDOM(assocBaseDoc);
            finally
              DOMImpl.freeDocument(assocBaseDoc);
            end; {try ... finally ...}
          end; {if ok ...}
        finally
          DOMImpl.freeDocument(dummyDoc);
          DomImpl.freeUnusedASModels;
        end; {try ... finally ...}
      end; {with ...}

    except
      result:= false;
    end;
  finally
    domImpl.free;
    docBuilder.free;
    standardDomReader.free;
    xmlToDomParser.free;
  end;
end;

function TApeMappingRepository.UpdateLocalCatalogs: boolean;
var
  ok1,ok2,ok3: boolean;
begin
  ok1:= UpdateLocalDisplayMappingCatalogFromURL;
  ok2:= UpdateLocalInfoFilesCatalogFromURL;
  ok3:= UpdateLocalResolveMappingCatalogFromURL;
  result:= ok1 and ok2 and ok3;
end;

function TApeMappingRepository.UpdateLocalDisplayMappingCatalog(
  const MappingCatalog: TApeMappingCatalog): boolean;
begin
  FDisplayMappingCatalog.Clear;
  FDisplayMappingCatalog.Assign(MappingCatalog);
  result:= true;
end;

function TApeMappingRepository.UpdateLocalDisplayMappingCatalogFromURL: boolean;
var
  URL: wideString;
begin
  FDisplayMappingCatalog.Clear;
  if assigned(UserPreferences) then begin
    URL:= FilenameToUriStr(UserPreferences.ActiveDisplayMappingDoc,[fuPlainColon]);
    result:= UpdateMappingCatalogFromURL(URL,FDisplayMappingCatalog);
  end else result:= false;
end;

function TApeMappingRepository.UpdateLocalInfoFilesCatalog(
  const MappingCatalog: TApeMappingCatalog): boolean;
begin
  FInfoFilesCatalog.Clear;
  FInfoFilesCatalog.Assign(MappingCatalog);
  result:= true;
end;

function TApeMappingRepository.UpdateLocalInfoFilesCatalogFromURL: boolean;
var
  URL: wideString;
begin
  FInfoFilesCatalog.Clear;
  if assigned(UserPreferences) then begin
    URL:= FilenameToUriStr(UserPreferences.ActiveInfoFilesDoc,[fuPlainColon]);
    result:= UpdateMappingCatalogFromURL(URL,FInfoFilesCatalog);
  end else result:= false;
end;

function TApeMappingRepository.UpdateLocalResolveMappingCatalog(
  const MappingCatalog: TApeMappingCatalog): boolean;
begin
  FResolveMappingCatalog.Clear;
  FResolveMappingCatalog.Assign(MappingCatalog);
  result:= true;
end;

function TApeMappingRepository.UpdateLocalResolveMappingCatalogFromURL: boolean;
var
  URL: wideString;
begin
  FResolveMappingCatalog.Clear;
  if assigned(UserPreferences) then begin
    URL:= FilenameToUriStr(UserPreferences.ActiveResolveMappingDoc,[fuPlainColon]);
    result:= UpdateMappingCatalogFromURL(URL,FResolveMappingCatalog);
  end else result:= false;
end;

{ TApeUrnRepository }

procedure TApeUrnRepository.FindDisplayURLs(const urn: wideString;
                                            const UrlList: TdomNameValueList);
begin
  if (urn = '') or not assigned(UrlList)
    then exit;
  if assigned(FMappingRepository)
    then FMappingRepository.FindDisplayURLs(urn,UrlList);
end;

function TApeUrnRepository.FindTargetMappingLabel(const urn: wideString): wideString;
begin
  if (urn = '') or not assigned(FMappingRepository)
    then result:= ''
    else result:= FMappingRepository.FindTargetMappingLabel(urn);
end;

function TApeUrnRepository.FindURNLabel(const urn: wideString): wideString;
begin
  if (urn = '') or not assigned(FMappingRepository)
    then result:= ''
    else result:= FMappingRepository.FindURNLabel(urn);
end;

procedure TApeUrnRepository.Notification(AComponent: TComponent;
                                         Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then begin
    if AComponent = FMappingRepository then begin
      FMappingRepository:= nil;
    end;
  end;
end;

function TApeUrnRepository.OpenCatalogAdminDlg: boolean;
begin
  if assigned(FMappingRepository) then begin
    FMappingRepository.OpenCatalogAdminDlg;
    result:= true;
  end else result:= false;
end;

function TApeUrnRepository.RequestDisplayPacket(const urnList: TdomWideStringList): TApeDisplayPacket;
begin
  if assigned(FMappingRepository)
    then result:= FMappingRepository.RequestDisplayPacket(urnList)
    else result:= nil;
end;

procedure TApeUrnRepository.ResolveAndDisplayURN(const urn: wideString;
  const UrlList: TdomNameValueList);
var
  index: integer;
  ResolvableUrnsList: TdomNameValueList;
begin
  if (urn = '') or not assigned(UrlList) then exit;

  ResolvableUrnsList:= TdomNameValueList.Create;
  try
    ResolveUrn(urn,ResolvableUrnsList);
    UrlList.clear;
    for index:= 0 to pred(ResolvableUrnsList.length) do
      FindDisplayURLs(ResolvableUrnsList.names[index],UrlList);
  finally
    ResolvableUrnsList.Free;
  end;
end;

procedure TApeUrnRepository.ResolveUrn(const urn: wideString;
  const DisplayableUrnsList: TdomNameValueList);
begin
  if (urn = '') or not assigned(DisplayableUrnsList)
    then exit;
  if assigned(FMappingRepository)
    then FMappingRepository.FindResolveURNs(urn,DisplayableUrnsList);
end;

procedure TApeUrnRepository.SetMappingRepository(
  const Value: TApeMappingRepository);
begin
  FMappingRepository:= Value;
end;

end.
