unit ApePreferences;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "ApePreferences.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  XDOM_2_4, ApeUtils, Dialogs,
  Forms, Graphics, Classes, Contnrs, Controls, IniFiles, SysUtils;

type
  TActiveUserChangingEvent = procedure(    Sender: TObject;
                                           UserIndex: Integer;
                                       var AllowChange: Boolean) of object;

  TProfileManagerStartOpt = (soNormal, soInitializing);

  TApeUser = class
  private
    FDirectory: string;
    FMemIniFile: TMemIniFile;
    FProfileName: string;
    function GetDemoOpt: boolean;
    function GetDisplayMappingDoc: TFilename;
    function GetFontName: TFontName;
    function GetInfoFilesDoc: TFilename;
    function GetMemoryCacheSize: integer;
    function GetName: wideString;
    function GetResolveMappingDoc: TFilename;
    function GetUserDirectory: TFilename;
    procedure SetFontName(const Value: TFontName);
    procedure SetMemoryCacheSize(const Value: integer);
    procedure SetName(const Str: wideString);
  public
    constructor Create(const Directory,
                             ProfileName: string); virtual;
    destructor Destroy; override;
    destructor DestroyInfoFile; virtual;
    function Rename(const ProfileName: string): boolean; virtual;
    property DemoOpt: boolean read GetDemoOpt;
    property DisplayMappingDoc: TFilename read GetDisplayMappingDoc;
    property FontName: TFontName read GetFontName write SetFontname;
    property InfoFilesDoc: TFilename read GetInfoFilesDoc;
    property MemIniFile: TMemIniFile read FMemIniFile;
    property MemoryCacheSize: integer read GetMemoryCacheSize write SetMemoryCacheSize;
    property Name: wideString read GetName write SetName;
    property ProfileName: string read FProfileName;
    property ResolveMappingDoc: TFilename read GetResolveMappingDoc;
    property UserDirectory: TFilename read GetUserDirectory;
  end;

  TApePreferences = class(TComponent)
  private
    FChange: TNotifyEvent;
    FChanging: TActiveUserChangingEvent;
    FActiveUserIndex: integer;
    FTemplateDirectory: TFilename;
    FUsers: TObjectList;
    FUsersDirectory: TFilename;
    function GetActiveDisplayMappingDoc: TFilename;
    function GetActiveInfoFilesDoc: TFilename;
    function GetActiveMemIniFile: TMemIniFile;
    function GetActiveResolveMappingDoc: TFilename;
    function GetActiveUser: TApeUser;
    function GetActiveUserDirectory: TFilename;
    function GetDemoMode: boolean;
    function GetUserCount: Integer;
    function GetUsers(Index: Integer): TApeUser;
    procedure SetActiveUserIndex(const Value: Integer);
    procedure SetUsersDirectory(const Value: TFilename);
  protected
    function Add(const ProfileName: string): integer; virtual;
    procedure DoChange; virtual;
    function DoChanging(const UserIndex: Integer): Boolean; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ActivateUser(const User: TApeUser): boolean; virtual;
    procedure Assign(Source: TPersistent); override;
    procedure AssignTo(Dest: TPersistent); override;
    function DeleteInfoFile(const ProfileName: string): boolean; virtual;
    function FindUser(const ProfileName: string): integer; virtual;
    function Rename(const OldProfileName,
                          NewProfileName: string): boolean; virtual;
    function ShowEditProfileDialog(const Index: Integer): boolean; virtual;
    function ShowProfileManager(const Opt: TProfileManagerStartOpt): boolean; virtual;
    procedure Update; virtual;
    property ActiveDisplayMappingDoc: TFilename read GetActiveDisplayMappingDoc;
    property ActiveInfoFilesDoc: TFilename read GetActiveInfoFilesDoc;
    property ActiveMemIniFile: TMemIniFile read GetActiveMemIniFile;
    property ActiveResolveMappingDoc: TFilename read GetActiveResolveMappingDoc;
    property ActiveUser: TApeUser read GetActiveUser;
    property ActiveUserDirectory: TFilename read GetActiveUserDirectory;
    property ActiveUserIndex: Integer read FActiveUserIndex write SetActiveUserIndex;
    property DemoMode: boolean read GetDemoMode;
    property UserCount: Integer read GetUserCount;
    property Users[Index: Integer]: TApeUser read GetUsers;
  published
    property TemplateDirectory: TFilename read FTemplateDirectory write FTemplateDirectory;
    property UsersDirectory: TFilename read FUsersDirectory write SetUsersDirectory;

    property OnChange: TNotifyEvent read FChange write FChange;
    property OnChanging: TActiveUserChangingEvent read FChanging write FChanging;
  end;

procedure Register;

implementation

uses
  ApeEditProfileDlg, ApeProfileManagerDlg, ApeUserInterface;

type
  TSpecialMemoryStream = class(TMemoryStream)
  public
    procedure SetPointer(Ptr: Pointer; Size: Longint);
  end;

procedure Register;
begin
  RegisterComponents('Ape', [ TApePreferences
                            ]);
end;

{ TSpecialMemoryStream }

procedure TSpecialMemoryStream.SetPointer(Ptr: Pointer; Size: Integer);
begin
  inherited setPointer(ptr,size);
end;

{ TApeUser }

constructor TApeUser.Create(const Directory,
                                  ProfileName: string);
begin
  if Directory = ''
    then raise EInvalidOperation.Create('Directory not specified.');
  if not DirectoryExists(Directory)
    then raise EInvalidOperation.Create('Directory does not exist.');
  if ProfileName = ''
    then raise EInvalidOperation.Create('ProfileName not specified.');
  FDirectory:= Directory;
  if FDirectory[Length(FDirectory)] <> '\'
    then FDirectory := concat(FDirectory,'\');
  FProfileName:= ProfileName;
  FMemIniFile:= TMemIniFile.Create(UserDirectory + 'settings.ini');
end;

destructor TApeUser.Destroy;
begin
  FMemIniFile.UpdateFile; // Save changes.
  FMemIniFile.free;
  inherited Destroy;
end;

destructor TApeUser.DestroyInfoFile;
begin
  DeleteFile(DisplayMappingDoc);
  DeleteFile(InfoFilesDoc);
  DeleteFile(ResolveMappingDoc);
  DeleteFile(MemIniFile.FileName);
  RemoveDir(UserDirectory+'catalogs');
  RemoveDir(UserDirectory);

  inherited Destroy;
end;

function TApeUser.GetDemoOpt: boolean;
begin
  result:= FMemIniFile.ReadBool('Options','demo',false);
end;

function TApeUser.GetDisplayMappingDoc: TFilename;
begin
  if UserDirectory <> ''
    then result:= UserDirectory + 'catalogs\display_maps.xml'
    else result := '';
end;

function TApeUser.GetFontName: TFontName;
begin
  result:= FMemIniFile.ReadString('DisplayFont','fontName',DEFAULT_FONT_NAME);
end;

function TApeUser.GetInfoFilesDoc: TFilename;
begin
  if UserDirectory <> ''
    then result:= UserDirectory + 'catalogs\info_files.xml'
    else result := '';
end;

function TApeUser.GetMemoryCacheSize: integer;
begin
  result:= FMemIniFile.ReadInteger('Cache','memoryCacheSize',MEMORY_CACHE_SIZE_MIN);
  if result < MEMORY_CACHE_SIZE_MIN then begin
    SetMemoryCacheSize(MEMORY_CACHE_SIZE_MIN);
    result:= MEMORY_CACHE_SIZE_MIN;
  end;
end;

function TApeUser.GetName: wideString;
var
  MStream: TdomWideStringStream;
begin
  MStream:= TdomWideStringStream.create('');
  try
    FMemIniFile.ReadBinaryStream('Identitiy','name',MStream);
    result:= MStream.dataString;
  finally
    MStream.free;
  end;
end;

function TApeUser.GetResolveMappingDoc: TFilename;
begin
  if UserDirectory <> ''
    then result:= UserDirectory + 'catalogs\resolve_maps.xml'
    else result := '';
end;

function TApeUser.GetUserDirectory: TFilename;
begin
  result:= FDirectory + FProfileName + '\';
end;

function TApeUser.Rename(const ProfileName: string): boolean;
var
  newProfilePath: TFilename;
  oldProfilePath: TFilename;
begin
  if ProfileName = ''
    then begin result:= false; exit; end;
  result:= true;
  if FProfileName = ProfileName then exit;

  oldProfilePath:= FDirectory + FProfileName;
  newProfilePath:= FDirectory + ProfileName;

  FProfileName:= ProfileName;
  FMemIniFile.UpdateFile; // Save changes.
  FMemIniFile.free;

  RenameFile(oldProfilePath,newProfilePath);

  FMemIniFile:= TMemIniFile.Create(UserDirectory + 'settings.ini');
end;

procedure TApeUser.SetFontName(const Value: TFontName);
begin
  if value = ''
    then FMemIniFile.WriteString('DisplayFont','fontName',DEFAULT_FONT_NAME)
    else FMemIniFile.WriteString('DisplayFont','fontName',Value);
end;

procedure TApeUser.SetMemoryCacheSize(const Value: integer);
begin
  if Value < MEMORY_CACHE_SIZE_MIN
    then FMemIniFile.WriteInteger('Cache','memoryCacheSize',MEMORY_CACHE_SIZE_MIN)
    else FMemIniFile.WriteInteger('Cache','memoryCacheSize',Value);
end;

procedure TApeUser.SetName(const Str: wideString);
var
  ptr: PChar;
  size: longint;
  MStream: TSpecialMemoryStream;
begin
  ptr:= pointer(str);
  size:= length(str)*2;
  MStream:= TSpecialMemoryStream.create;
  try
    MStream.SetPointer(ptr,size);
    FMemIniFile.WriteBinaryStream('Identitiy','name',MStream);
  finally
    MStream.free;
  end;
end;

{ TApePreferences }

constructor TApePreferences.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FUsers := TObjectList.Create;
  FActiveUserIndex:= -1;
end;

destructor TApePreferences.Destroy;
begin
  FUsers.Free;
  inherited Destroy;
end;

function TApePreferences.ActivateUser(const User: TApeUser): boolean;
var
  i: integer;
begin
  result:= false;
  for i:= 0 to pred(UserCount) do
    if Users[i] = User then begin
      ActiveUserIndex:= i;
      result:= true;
      break;
    end;
end;

function TApePreferences.Add(const ProfileName: string): integer;
begin
  Result:= FUsers.Add(TApeUser.create(UsersDirectory,ProfileName));
end;

function TApePreferences.DeleteInfoFile(const ProfileName: string): boolean;
var
  index: integer;
  profile: TApeUser;
  UserDirectory: TFilename;
begin
  index:= FindUser(ProfileName);
  if index = -1 then begin result:= false; exit; end;
  profile:= Users[index];
  FUsers.Extract(profile);
  if index < FActiveUserIndex then begin
    dec(FActiveUserIndex);
  end else if index = FActiveUserIndex then begin
    FActiveUserIndex:= -1;
  end;
  UserDirectory:= profile.UserDirectory;
  profile.DestroyInfoFile;
  result:= not FileExists(UserDirectory);
end;

procedure TApePreferences.DoChange;
begin
  if assigned(FChange)
    then FChange(self);
end;

function TApePreferences.DoChanging(const UserIndex: Integer): Boolean;
begin
  result:= true;
  if assigned(FChanging)
    then FChanging(self,UserIndex,result);
end;

function TApePreferences.FindUser(const ProfileName: string): integer;
var
  i: integer;
begin
  result:= -1;
  for i:= 0 to pred(UserCount) do
    if Users[i].ProfileName = ProfileName then begin
      result:= i;
      break;
    end;
end;

function TApePreferences.Rename(const OldProfileName,
                                          NewProfileName: string): boolean;
var
  oldIndex, newIndex: integer;
begin
  oldIndex:= FindUser(OldProfileName);
  newIndex:= FindUser(NewProfileName);
  if (oldIndex = -1) or (newIndex > -1)
    then begin result:= false; exit; end;
  result:= Users[oldIndex].Rename(NewProfileName);
end;

function TApePreferences.GetActiveDisplayMappingDoc: TFilename;
begin
  if ActiveUserIndex > -1
    then result:= Users[ActiveUserIndex].DisplayMappingDoc
    else result:= '';
end;

function TApePreferences.GetActiveInfoFilesDoc: TFilename;
begin
  if ActiveUserIndex > -1
    then result:= Users[ActiveUserIndex].InfoFilesDoc
    else result:= '';
end;

function TApePreferences.GetActiveMemIniFile: TMemIniFile;
begin
  if ActiveUserIndex > -1
    then result:= Users[ActiveUserIndex].MemIniFile
    else result:= nil;
end;

function TApePreferences.GetActiveResolveMappingDoc: TFilename;
begin
  if ActiveUserIndex > -1
    then result:= Users[ActiveUserIndex].ResolveMappingDoc
    else result:= '';
end;

function TApePreferences.GetActiveUser: TApeUser;
begin
  if ActiveUserIndex > -1
    then result:= Users[ActiveUserIndex]
    else result:= nil;
end;

function TApePreferences.GetActiveUserDirectory: TFilename;
begin
  if ActiveUserIndex > -1
    then result:= Users[ActiveUserIndex].UserDirectory
    else result:= '';
end;

function TApePreferences.GetDemoMode: boolean;
begin
  if ActiveUserIndex > -1
    then result:= Users[ActiveUserIndex].DemoOpt
    else result:= false;
end;

function TApePreferences.GetUserCount: Integer;
begin
  Result:= FUsers.Count;
end;

function TApePreferences.GetUsers(Index: Integer): TApeUser;
begin
  Result:= TApeUser(FUsers[Index]);
end;


procedure TApePreferences.SetActiveUserIndex(const Value: Integer);
begin
  if (value < -1) or (value > pred(UserCount))
    then raise EInvalidOperation.Create('ActiveUserIndex out of bounds.');
  if FActiveUserIndex <> Value then begin
    if DoChanging(Value) then begin
      FActiveUserIndex:= Value;
      DoChange;
    end;
  end;
end;

procedure TApePreferences.SetUsersDirectory(const Value: TFilename);
begin
  if FUsersDirectory = Value then exit;
  FUsersDirectory:= Value;
  Update;
end;

function TApePreferences.ShowEditProfileDialog(const Index: Integer): boolean;
var
  editProfileDialog: TEditProfileDialog;
begin
  editProfileDialog:= TEditProfileDialog.Create(self);
  try
    result:= ( editProfileDialog.ShowModal(Users[Index]) = mrOK );
  finally
    editProfileDialog.Free;
  end;
end;

function TApePreferences.ShowProfileManager(const Opt: TProfileManagerStartOpt): boolean;
var
  newProfileManager: TProfileManager;
begin
  newProfileManager:= TProfileManager.Create(self);
  try
    case Opt of
      soNormal: newProfileManager.State:= pmNormal;
      soInitializing: newProfileManager.State:= pmInitializing;
    end;
    result:= ( newProfileManager.ShowModal(self) = mrOK );
  finally
    newProfileManager.Free;
  end;
end;

procedure TApePreferences.Update;
var
  F: TSearchRec;
  Error: integer;
begin
  FUsers.Clear;
  if UsersDirectory = '' then exit;

  Error:= FindFirst(UsersDirectory+'*.*',faDirectory,F);
  try
    while Error = 0 do begin
      if (F.Name <> '.') and (F.Name <> '..')
        then Add(F.Name);
      Error:= FindNext(F);
    end;
  finally
    FindClose(F);
  end;
end;

procedure TApePreferences.AssignTo(Dest: TPersistent);
var
  t,l,h,w: Integer;
begin
  if Dest is TUserInterface then begin
    with (Dest as TUserInterface) do begin
      if ActiveUserIndex = -1 then begin
        Caption:= 'APE';
        ApeMappingRepository.CacheSize:= MEMORY_CACHE_SIZE_MIN;
      end else begin

        // Updates the form's caption to display the profile name:
        Caption:= concat('APE [',Users[ActiveUserIndex].ProfileName,']');

        // Updates the preferences
        with ActiveMemIniFile do begin
          if SectionExists('Screen') then begin

            Position:= poDesigned;
            t:= ReadInteger('Screen','top',Top);
            l:= ReadInteger('Screen','left',Left);
            h:= ReadInteger('Screen','height',Height);
            w:= ReadInteger('Screen','width',Width);
            UpdateBoundsRect(Rect(l,t,l+w,t+h));

            if ReadBool('Screen','maximized',false)
              then WindowState:= wsMaximized
              else WindowState:= wsNormal;

            StructureDisplay.Width:= ReadInteger('Screen','structureDisplayWidth',StructureDisplay.Width);
            SecondaryDisplay.Height:= ReadInteger('Screen','secondaryDisplayHeight',SecondaryDisplay.Height);

          end else Position:= poScreenCenter;
        end; {with ...}

        // Updates the cache properties:
        with ApeMappingRepository do begin
          CacheSize:= ActiveUser.MemoryCacheSize;
          ClearCache;
        end; {with ...}

        // Update Primary Display font:
        PrimaryDisplay.Font.Name:= ActiveUser.FontName;

        // Update Secondary Display font:
        SecondaryDisplay.ApparatusFontName:= ActiveUser.FontName;
        SecondaryDisplay.UpdateComment;

        // Update Structure Display font:
        StructureDisplay.DisplayFontName:= ActiveUser.FontName;

        // Switch to demo mode:
        MainMenu.Items[1].visible:= not DemoMode;
        if DemoMode and (ActiveUserDirectory <> '')
          then StructureDisplay.OpenDirectory(ActiveUserDirectory);
      end; {if ... else ...}
    end; {with ...}
  end else inherited AssignTo(Dest);
end;

procedure TApePreferences.Assign(Source: TPersistent);
begin
  if Source is TUserInterface then begin
    if ActiveUserIndex > -1 then
      with (Source as TUserInterface) do 
        with ActiveMemIniFile do begin

          if WindowState = wsMaximized
            then WriteBool('Screen','maximized',true)
            else begin
              WriteBool('Screen','maximized',false);
              WriteInteger('Screen','top',Top);
              WriteInteger('Screen','left',Left);
              WriteInteger('Screen','height',Height);
              WriteInteger('Screen','width',Width);
            end;

          WriteInteger('Screen','structureDisplayWidth',StructureDisplay.Width);
          WriteInteger('Screen','secondaryDisplayHeight',SecondaryDisplay.Height);

        end;
  end else inherited Assign(Source);
end;

end.
