unit ApeProfileManagerDlg;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "ApeProfileManagerDlg.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  ApePreferences,
  SysUtils, Classes, Forms, Dialogs, ExtCtrls, StdCtrls, Controls;

type
  TProfileManagerState = (pmNormal,pmInitializing);

  TProfileManager = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    GroupBox1: TGroupBox;
    ProfileList: TListBox;
    Panel1: TPanel;
    TitleLabel: TLabel;
    SubtitleLabel: TLabel;
    CreateProfileBtn: TButton;
    RenameProfileBtn: TButton;
    DeleteProfilBtn: TButton;
    EditProfileBtn: TButton;
    procedure CreateProfileBtnClick(Sender: TObject);
    procedure DeleteProfilBtnClick(Sender: TObject);
    procedure EditProfileBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure RenameProfileBtnClick(Sender: TObject);
  private
    FUserPreferences: TApePreferences;
    FState: TProfileManagerState;
    procedure SetState(const Value: TProfileManagerState);
  protected
    procedure UpdateProfileList(const SelectionName: string); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    function ShowModal(UserPreferences: TApePreferences): Integer; reintroduce; overload; virtual;
    property State: TProfileManagerState  read FState write SetState default pmNormal;
  end;

var
  ProfileManager: TProfileManager;

implementation

uses
  ApeDeleteProfileDlg, ApeEditProfileDlg, ApeNewProfileDlg, ApeRenameProfileDlg;

{$R *.dfm}

{TProfileManager}

constructor TProfileManager.Create(AOwner: TComponent);
begin
  inherited;
  FUserPreferences:= nil;
  State:= pmNormal;
end;

procedure TProfileManager.UpdateProfileList(const SelectionName: string);
var
  i: integer;
begin
  with ProfileList do begin
    Clear;
    with FUserPreferences do
      for i:= 0 to pred(UserCount) do
        AddItem(Users[i].ProfileName,Users[i]);
    for i:= 0 to pred(Items.Count) do begin
      if Items.Strings[i] = SelectionName then begin
        ItemIndex:= i;
        break;
      end;
    end;
    if ItemIndex = -1
      then ItemIndex:= 0;
  end;
end;

procedure TProfileManager.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if ModalResult = mrOK
    then if assigned(FUserPreferences)
      then with ProfileList do
        if ItemIndex > -1
          then FUserPreferences.ActivateUser(TApeUser(Items.Objects[ItemIndex]));
end;

procedure TProfileManager.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if assigned(FUserPreferences)
    then if FUserPreferences.UserCount = 0
      then if ModalResult = mrOK then begin
        ShowMessage('In order to run APE you must set up at least one profile.');
        CanClose:= false;
      end;
end;

procedure TProfileManager.CreateProfileBtnClick(Sender: TObject);
var
  newProfileDialog: TNewProfileDialog;
begin
  newProfileDialog:= TNewProfileDialog.Create(self);
  try
    newProfileDialog.UserPreferences:= FUserPreferences;
    if newProfileDialog.ShowModal = mrOK then begin
      CreateDir(newProfileDialog.ProfilePath);
      FUserPreferences.Update;
      UpdateProfileList(newProfileDialog.ProfileNameEdit.Text);
      EditProfileBtnClick(Sender);
    end;
  finally
    newProfileDialog.Free;
  end;
end;

procedure TProfileManager.DeleteProfilBtnClick(Sender: TObject);
var
  deleteProfileDialog: TDeleteProfileDialog;
  profileName: string;
begin
  if ProfileList.ItemIndex = -1 then exit;
  profileName:= ProfileList.Items.Strings[ProfileList.ItemIndex];

  deleteProfileDialog:= TDeleteProfileDialog.Create(self);
  try

    deleteProfileDialog.OldProfileName:= profileName;

    if deleteProfileDialog.ShowModal = mrOK then begin
      if not FUserPreferences.DeleteInfoFile(profileName)
        then showMessage('Some directories could not be deleted, because they are containing unidentified files.  Please check "'+
                         FUserPreferences.UsersDirectory + '" and remove them manually.');
      UpdateProfileList('');
    end;

  finally
    deleteProfileDialog.Free;
  end;
end;

procedure TProfileManager.EditProfileBtnClick(Sender: TObject);
var
  editProfileDialog: TEditProfileDialog;
begin
  if ProfileList.ItemIndex = -1 then exit;

  editProfileDialog:= TEditProfileDialog.Create(self);
  try
    editProfileDialog.ShowModal(FUserPreferences.Users[FUserPreferences.FindUser(ProfileList.Items.Strings[ProfileList.ItemIndex])]);
  finally
    editProfileDialog.Free;
  end;
end;

procedure TProfileManager.RenameProfileBtnClick(Sender: TObject);
var
  renameProfileDialog: TRenameProfileDialog;
  oldProfileName: string;
begin
  if ProfileList.ItemIndex = -1 then exit;

  renameProfileDialog:= TRenameProfileDialog.Create(self);
  try
    oldProfileName:= ProfileList.Items.Strings[ProfileList.ItemIndex];
    renameProfileDialog.UserPreferences:= FUserPreferences;
    renameProfileDialog.OldProfileName:= oldProfileName;
    if renameProfileDialog.ShowModal = mrOK then begin
      FUserPreferences.Rename(oldProfileName,renameProfileDialog.ProfileNameEdit.Text);
      UpdateProfileList(renameProfileDialog.ProfileNameEdit.Text);
    end;
  finally
    renameProfileDialog.Free;
  end;
end;

procedure TProfileManager.SetState(const Value: TProfileManagerState);
begin
  FState:= Value;
  if FState = pmNormal then begin
    Caption:= 'Manage User Profiles';
    OKBtn.Caption:= 'Finish';
    CancelBtn.Visible:= false;
  end else begin
    Caption:= 'Select User Profile';
    OKBtn.Caption:= 'Start APE';
    CancelBtn.Visible:= true;
  end;
end;

function TProfileManager.ShowModal(UserPreferences: TApePreferences): Integer;
begin
  if not assigned(UserPreferences)
    then raise EInvalidOperation.Create('UserPreferences not specified.');
  if UserPreferences.UsersDirectory = ''
    then raise EInvalidOperation.Create('UserPreferences.Directory not specified.');

  FUserPreferences:= UserPreferences;
  try
    with FUserPreferences do
      if ActiveUserIndex = -1
        then UpdateProfileList('')
        else UpdateProfileList(Users[ActiveUserIndex].ProfileName);

    result:= inherited ShowModal;
  finally
    FUserPreferences:= nil;
  end;
end;

end.
