unit ApeRepositoryViews;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "ApeRepositoryViews.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  ApeCaches, VirtualTrees, Controls, ComCtrls, Classes;

type

  TABViewTreeData = record
    SourceIndex: integer;  // The index of the SourceURN in the related TApeMappingCatalog
    TargetIndex: integer;  // The index of the TargetURN in the related TApeMappingCatalog
                           // or -1 if we are refering to the whole SourceURN list
  end;

  PABViewTreeData = ^TABViewTreeData;

  TApeMappingCatalogView = class(TCustomControl)
  private
    FMappingCatalog: TApeMappingCatalog;
    FVirtualStringTree: TVirtualStringTree;
    procedure VirtualStringTreeGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    function GetTargetFocused: boolean;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    function Add(const targetMappingURN,
                       sourceMappingURN,
                       targetURL,
                       targetTitle: wideString): boolean; virtual;
    function DeleteFocusedTarget: boolean; virtual;
    procedure UpdateView(const MappingCatalog: TApeMappingCatalog); virtual;
    property MappingCatalog: TApeMappingCatalog read FMappingCatalog;
    property TargetFocused: boolean read GetTargetFocused;
  published
    property Align;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ape', [TApeMappingCatalogView
                            ]);
end;

{ TApeMappingCatalogView }

constructor TApeMappingCatalogView.Create(aOwner: TComponent);
var
  newColumn: TVirtualTreeColumn;
begin
  inherited;
  FMappingCatalog:= TApeMappingCatalog.Create;

  FVirtualStringTree:= TVirtualStringTree.Create(self);
  with FVirtualStringTree do begin
    Align:= alClient;
    OnGetText:= VirtualStringTreeGetText;
    Parent:= self;

    with Header do begin
      AutoSizeIndex:= 1;
      with Columns do begin

        newColumn:= Add;
        with newColumn do begin
          MinWidth:= 150;
          Options:= [ coAllowClick,
                      coDraggable,
                      coEnabled,
                      coParentBidiMode,
                      coParentColor,
                      coResizable,
                      coShowDropMark,
                      coVisible
                    ];
          Text:= 'URI';
          Width:= 250;
        end;

        newColumn:= Add;
        with newColumn do begin
          MinWidth:= 150;
          Options:= [ coAllowClick,
                      coDraggable,
                      coEnabled,
                      coParentBidiMode,
                      coParentColor,
                      coResizable,
                      coShowDropMark,
                      coVisible
                    ];
          Text:= 'Title';
        end;
      end;
      Options:= [ hoAutoResize,
                  hoColumnResize,
                  hoDblClickResize,
                  hoVisible];
    end;
  end;
end;

function TApeMappingCatalogView.Add(const targetMappingURN,
                                          sourceMappingURN,
                                          targetURL,
                                          targetTitle: wideString): boolean;
begin
  result:= true;
  FMappingCatalog.Append(targetMappingURN,sourceMappingURN,targetURL,targetTitle);
  UpdateView(FMappingCatalog);
end;

function TApeMappingCatalogView.DeleteFocusedTarget: boolean;
var
  data: PABViewTreeData;
  i: integer;
  f_source_index: integer;
  f_target_index: integer;
  fn: PVirtualNode;
  source_index: integer;
  target_index: integer;
begin
  Result:= TargetFocused;
  if Result then begin
    with FVirtualStringTree do begin
      BeginUpdate;
      try
        // Calculate new focused node:
        with FocusedNode^ do begin
          fn:= PrevSibling;
          if not assigned(fn) then begin
            fn:= NextSibling;
            if not assigned(fn) then begin
              if assigned(Parent) then begin
                fn:= Parent.PrevSibling;
                if not assigned(fn)
                  then if assigned(Parent.NextSibling)
                    then fn:= Parent;
              end;
            end else fn:= FocusedNode;
          end;
        end;

        // Bookmark the index of the new focus
        if assigned(fn) then begin
          data:= GetNodeData(fn);
          f_source_index:= data.SourceIndex;
          f_target_index:= data.TargetIndex;
        end else begin
          f_source_index:= -1;
          f_target_index:= -1;
        end;

        // Delete the focused node
        data:= GetNodeData(FocusedNode);
        source_index:= data.SourceIndex;
        target_index:= data.TargetIndex;
        FMappingCatalog.RemoveByIndex(source_index,target_index);
        UpdateView(FMappingCatalog);

        // Find new focus node in new Tree:
        if f_source_index > -1 then begin
          fn:= GetFirst;
          for i:= 1 to f_source_index do
            fn:= GetNextSibling(fn);
          if f_target_index > -1 then begin
            fn:= GetFirstChild(fn);
            for i:= 1 to f_target_index do
              fn:= GetNextSibling(fn);
          end;
          Expanded[fn]:= true;
          ScrollIntoView(fn,true,true);
          FocusedNode:= fn;
          Selected[fn]:= True;
        end;

      finally
        EndUpdate;
      end; {try ... finally ...}
    end;
  end;
end;

destructor TApeMappingCatalogView.Destroy;
begin
  FMappingCatalog.free;
  inherited;
end;

function TApeMappingCatalogView.GetTargetFocused: boolean;
var
  data: PABViewTreeData;
  target_index: integer;
  tn: PVirtualNode;
begin
  tn:= FVirtualStringTree.FocusedNode;
  if assigned(tn) then begin
    data:= FVirtualStringTree.GetNodeData(tn);
    target_index:= data.TargetIndex;
    if target_index > -1
      then result:= true
      else result:= false;
  end else result:= false;
end;

procedure TApeMappingCatalogView.UpdateView(const MappingCatalog: TApeMappingCatalog);
var
  data: PABViewTreeData;
  i: integer;
  j: integer;
  tn_i: PVirtualNode;
  tn_j: PVirtualNode;
begin
  if assigned(MappingCatalog)
    then FMappingCatalog.Assign(MappingCatalog)
    else FMappingCatalog.Clear;
  with FVirtualStringTree do begin
    BeginUpdate;
    try
      Clear;
      NodeDataSize := SizeOf(TABViewTreeData);
      with FMappingCatalog do begin
        for i:= 0 to pred(SourceMappingCount) do begin
          tn_i:= AddChild(nil);
          data:= GetNodeData(tn_i);
          data.SourceIndex:= i;
          data.TargetIndex:= -1;
          for j:= 0 to pred(TargetCountByIndex(i)) do begin
            tn_j:= AddChild(tn_i);
            data:= GetNodeData(tn_j);
            data.SourceIndex:= i;
            data.TargetIndex:= j;
          end;
        end;
      end; {with ...}
    finally
      EndUpdate;
    end; {try ... finally ...}
  end; {with VirtualStringTree ...}
end;

procedure TApeMappingCatalogView.VirtualStringTreeGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
var
  data: PABViewTreeData;
  source_index: integer;
  target_index: integer;
begin
  data:= Sender.GetNodeData(Node);
  source_index:= data.SourceIndex;
  target_index:= data.TargetIndex;
  case Column of
    0: begin
      if target_index = -1
        then CellText:= FMappingCatalog.SourceMappingURN(source_index)
        else CellText:= FMappingCatalog.TargetURLByIndex(source_index,target_index);
    end;
    1: begin
      if target_index = -1
        then CellText:= ''
        else CellText:= FMappingCatalog.TargetTitleByIndex(source_index,target_index);
    end;
  end;
end;

end.
