unit FileDescDisplayFrame;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with 
// the License. You may obtain a copy of the License at 
// "http://www.mozilla.org/MPL/"
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is "FileDescDisplayFrame.pas".
// 
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg, 
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
// 
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which 
// case the provisions of the GPL are applicable instead of those above. 
// If you wish to allow use of your version of this file only under the 
// terms of the GPL, and not to allow others to use your version of this 
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  XDOM_2_4,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TFileDescDispFrame = class(TFrame)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    TitleEdit: TEdit;
    LabelLabel: TLabel;
    LabelEdit: TEdit;
    AuthorLabel: TLabel;
    AuthorEdit: TEdit;
    GroupBox1: TGroupBox;
    AvailabilityMemo: TMemo;
    AvailabilityStatusLabel: TLabel;
    AvailabilityStatusComboBox: TComboBox;
    SourceDescMemo: TMemo;
    AboutUrnEdit: TEdit;
    Label1: TLabel;
    TabSheet5: TTabSheet;
    DisplayPriorityMemo: TMemo;
  private
    FDisplayFontName: TFontName;
    FLanguage: TIso639LanguageCode;
  protected
    procedure SetDisplayFontName(const Value: TFontName); virtual;
    procedure UpdateNotesStmt(const ape_notesStmt: TdomElement); virtual;
    procedure UpdatePublicationStmt(const ape_publicationStmt: TdomElement); virtual;
    procedure UpdateRenderingStmt(const ape_renderingStmt: TdomElement); virtual;
    procedure UpdateSourceDesc(const ape_sourceDesc: TdomElement); virtual;
    procedure UpdateTitleStmt(const ape_titleStmt: TdomElement); virtual;
  public
    procedure UpdateDisplay(const ape_fileDesc: TdomElement); virtual;
    property DisplayFontName: TFontName read FDisplayFontName write SetDisplayFontName;
    property Language: TIso639LanguageCode read FLanguage write FLanguage;
  end;

implementation

const
  xmlns_ape:   wideString  = 'http://www.philo.de/ape/test-001';
  xmlns_xml:   wideString  = 'http://www.w3.org/XML/1998/namespace';

{$R *.DFM}

{ TFileDescDispFrame }

procedure TFileDescDispFrame.SetDisplayFontName(const Value: TFontName);
begin
  if FDisplayFontName <> Value then begin
    FDisplayFontName:= Value;
    // Future versions may insert here code to modify the display font.
  end;
end;

procedure TFileDescDispFrame.UpdateDisplay(
  const ape_fileDesc: TdomElement);
var
  ape_renderingStmt,ape_titleStmt,ape_publicationStmt,ape_sourceDesc,ape_notesStmt: TdomElement;
begin
  ape_titleStmt:= nil;
  ape_publicationStmt:= nil;
  ape_sourceDesc:= nil;
  ape_notesStmt:= nil;
  ape_renderingStmt:= nil;

  if assigned(ape_fileDesc) then begin
    with ape_fileDesc do begin
      if (namespaceURI = xmlns_ape) and (localName = 'fileDesc') then begin
        ape_titleStmt:=       getFirstChildElementNS(xmlns_ape,'titleStmt');
        ape_publicationStmt:= getFirstChildElementNS(xmlns_ape,'publicationStmt');
        ape_sourceDesc:=      getFirstChildElementNS(xmlns_ape,'sourceDesc');
        ape_notesStmt:=       getFirstChildElementNS(xmlns_ape,'notesStmt');
        ape_renderingStmt:=   getFirstChildElementNS(xmlns_ape,'renderingStmt');
      end; {if ...}
    end; {with ...}
  end; {if ...}

  UpdateTitleStmt(ape_titleStmt);
  UpdatePublicationStmt(ape_publicationStmt);
  UpdateSourceDesc(ape_sourceDesc);
  UpdateNotesStmt(ape_notesStmt);
  UpdateRenderingStmt(ape_renderingStmt);
end;

procedure TFileDescDispFrame.UpdateNotesStmt(
  const ape_notesStmt: TdomElement);
var
  ape_note: TdomElement;
  ape_type: TdomAttr;
  newAboutUrn, sourceDescLanguage: wideString;
begin
  newAboutUrn:= '';

  if assigned(ape_notesStmt) then begin
    if (ape_notesStmt.localName = 'notesStmt') and
       (ape_notesStmt.namespaceURI = xmlns_ape) then begin

      // Traverse <ape:note ...> tags
      sourceDescLanguage:= 'x-';
      ape_note:= ape_notesStmt.getFirstChildElementNS(xmlns_ape,'note');
      while assigned(ape_note) do begin
        ape_type:= ape_note.getAttributeNodeNS(xmlns_ape,'type');
        if assigned(ape_type)
          then if ape_type.value = 'about'
            then newAboutUrn:= ape_note.textContent;
        ape_note:= ape_note.getNextSiblingElementNS(xmlns_ape,'note');
      end; {while ...}

    end; {if ...}
  end; {if assigned(ape_notesStmt) ...}

  // xxx string vs. wideString
  AboutUrnEdit.Text:= newAboutUrn;
end;

procedure TFileDescDispFrame.UpdatePublicationStmt(
  const ape_publicationStmt: TdomElement);
var
  ape_availability,ape_p: TdomElement;
  ape_status,xml_lang: TdomAttr;
  specifiedLanguage,currentLanguage,availabilityLanguage: wideString;
  newAvailability,newAvailabilityStatus: wideString;
  Iso639Info: TIso639Info;
begin
  newAvailability:= '';
  newAvailabilityStatus:= '';

  if assigned(ape_publicationStmt) then begin
    if (ape_publicationStmt.localName = 'publicationStmt') and
       (ape_publicationStmt.namespaceURI = xmlns_ape) then begin

      // Search for <ape:availability ...>
      ape_availability:= ape_publicationStmt.getFirstChildElementNS(xmlns_ape,'availability');
      if assigned(ape_availability) then begin

        ape_status:= ape_availability.getAttributeNodeNS(xmlns_ape,'status');
        if assigned(ape_status)
          then newAvailabilityStatus:= ape_status.value
          else newAvailabilityStatus:= '';

        // Traverse <ape:p ...> tags
        Iso639Info:= TIso639Info.create;
        try
          specifiedLanguage:= Iso639Info.codeToSymbol(FLanguage);
        finally
          Iso639Info.free;
        end; {try ...}
        availabilityLanguage:= 'x-';
        ape_p:= ape_availability.getFirstChildElementNS(xmlns_ape,'p');
        while assigned(ape_p) do begin
          xml_lang:= ape_p.getAttributeNodeNS(xmlns_xml,'lang');
          if assigned(xml_lang)
            then currentLanguage:= xml_lang.value
            else currentLanguage:= '';
          if (availabilityLanguage <> currentLanguage) and  // exclude duplicates
             ( (availabilityLanguage = 'x-') or
               (currentLanguage = specifiedLanguage) or
               ((currentLanguage = '') and (availabilityLanguage <> specifiedLanguage)) )
            then begin
            availabilityLanguage:= currentLanguage;
            newAvailability:= ape_p.textContent;
            if (availabilityLanguage = specifiedLanguage) then break;
          end;
          ape_p:= ape_p.getNextSiblingElementNS(xmlns_ape,'p');
        end; {while ...}

      end; {if assigned(ape_availability) ...}

    end; {if ...}
  end; {if assigned(ape_publicationStmt) ...}

  // xxx string vs. wideString
  AvailabilityMemo.Text:= newAvailability;
  AvailabilityStatusComboBox.Text:= newAvailabilityStatus;
end;

procedure TFileDescDispFrame.UpdateRenderingStmt(
  const ape_renderingStmt: TdomElement);
var
  ape_displayPriority,ape_displayRole: TdomElement;
  ape_uid: TdomAttr;
begin
  DisplayPriorityMemo.Lines.BeginUpdate;
  try
    DisplayPriorityMemo.Clear;
    if assigned(ape_renderingStmt) then begin
      if (ape_renderingStmt.localName = 'renderingStmt') and
         (ape_renderingStmt.namespaceURI = xmlns_ape) then begin

        // Search for <ape:displayPriority ...>
        ape_displayPriority:= ape_renderingStmt.getFirstChildElementNS(xmlns_ape,'displayPriority');
        if assigned(ape_displayPriority) then begin
          ape_displayRole:= ape_displayPriority.getFirstChildElementNS(xmlns_ape,'displayRole');
          while assigned(ape_displayRole) do begin
            ape_uid:= ape_displayRole.getAttributeNodeNS(xmlns_ape,'uid');
            if assigned(ape_uid) then begin
              DisplayPriorityMemo.Lines.Add(ape_uid.nodeValue);
            end;
            ape_displayRole:= ape_displayRole.getNextSiblingElementNS(xmlns_ape,'displayRole');
          end; {while ...}
        end; {if ...}
      end; {if ...}
    end; {if ...}
  finally
    DisplayPriorityMemo.Lines.EndUpdate;
  end;
end;

procedure TFileDescDispFrame.UpdateSourceDesc(
  const ape_sourceDesc: TdomElement);
var
  ape_p: TdomElement;
  xml_lang: TdomAttr;
  Iso639Info: TIso639Info;
  specifiedLanguage,currentLanguage,sourceDescLanguage,newSourceDesc: wideString;
begin
  newSourceDesc:= '';

  if assigned(ape_sourceDesc) then begin
    if (ape_sourceDesc.localName = 'sourceDesc') and
       (ape_sourceDesc.namespaceURI = xmlns_ape) then begin

      // Traverse <ape:p ...> tags
      Iso639Info:= TIso639Info.create;
      try
        specifiedLanguage:= Iso639Info.codeToSymbol(FLanguage);
      finally
        Iso639Info.free;
      end; {try ...}
      sourceDescLanguage:= 'x-';
      ape_p:= ape_sourceDesc.getFirstChildElementNS(xmlns_ape,'p');
      while assigned(ape_p) do begin
        xml_lang:= ape_p.getAttributeNodeNS(xmlns_xml,'lang');
        if assigned(xml_lang)
          then currentLanguage:= xml_lang.value
          else currentLanguage:= '';
        if (sourceDescLanguage <> currentLanguage) and  // exclude duplicates
           ( (sourceDescLanguage = 'x-') or
             (currentLanguage = specifiedLanguage) or
             ((currentLanguage = '') and (sourceDescLanguage <> specifiedLanguage)) )
          then begin
          sourceDescLanguage:= currentLanguage;
          newSourceDesc:= ape_p.textContent;
          if (sourceDescLanguage = specifiedLanguage) then break;
        end;
        ape_p:= ape_p.getNextSiblingElementNS(xmlns_ape,'p');
      end; {while ...}

    end; {if ...}
  end; {if assigned(ape_sourceDesc) ...}

  // xxx string vs. wideString
  SourceDescMemo.Text:= newSourceDesc;
end;

procedure TFileDescDispFrame.UpdateTitleStmt(
  const ape_titleStmt: TdomElement);
var
  ape_title,ape_author: TdomElement;
  ape_type,xml_lang: TdomAttr;
  newTitle,newLabel,newAuthor: wideString;
  Iso639Info: TIso639Info;
  specifiedLanguage,currentLanguage,labelLanguage,titleLanguage,authorLanguage: wideString;
begin
  newTitle:= '';
  newLabel:= '';
  newAuthor:= '';

  if assigned(ape_titleStmt) then begin
    if (ape_titleStmt.localName = 'titleStmt') and
       (ape_titleStmt.namespaceURI = xmlns_ape) then begin

      // Traverse <ape:title ...> tags
      Iso639Info:= TIso639Info.create;
      try
        specifiedLanguage:= Iso639Info.codeToSymbol(FLanguage);
      finally
        Iso639Info.free;
      end; {try ...}
      labelLanguage:= 'x-';
      titleLanguage:= 'x-';
      ape_title:= ape_titleStmt.getFirstChildElementNS(xmlns_ape,'title');
      while assigned(ape_title) do begin
        ape_type:= ape_title.getAttributeNodeNS(xmlns_ape,'type');

        if assigned(ape_type) then begin
          if ape_type.value = 'full' then begin
            xml_lang:= ape_title.getAttributeNodeNS(xmlns_xml,'lang');
            if assigned(xml_lang)
              then currentLanguage:= xml_lang.value
              else currentLanguage:= '';
            if (titleLanguage <> currentLanguage) and  // exclude duplicates
               ( (titleLanguage = 'x-') or
                 (currentLanguage = specifiedLanguage) or
                 ((currentLanguage = '') and (titleLanguage <> specifiedLanguage)) )
              then begin
              titleLanguage:= currentLanguage;
              newTitle:= ape_title.textContent;
              if (labelLanguage = specifiedLanguage) and (titleLanguage = specifiedLanguage) then break;
            end;
          end else if ape_type.value = 'label' then begin
            xml_lang:= ape_title.getAttributeNodeNS(xmlns_xml,'lang');
            if assigned(xml_lang)
              then currentLanguage:= xml_lang.value
              else currentLanguage:= '';
            if (labelLanguage <> currentLanguage) and  // exclude duplicates
               ( (labelLanguage = 'x-') or
                 (currentLanguage = specifiedLanguage) or
                 ((currentLanguage = '') and (labelLanguage <> specifiedLanguage)) )
              then begin
              labelLanguage:= currentLanguage;
              newLabel:= ape_title.textContent;
              if (labelLanguage = specifiedLanguage) and (titleLanguage = specifiedLanguage) then break;
            end;
          end; {if ... else ...}
        end; {if assigned(ape_type) ...}

        ape_title:= ape_title.getNextSiblingElementNS(xmlns_ape,'title');
      end; {while ...}

      // Traverse <ape:author ...> tags
      authorLanguage:= 'x-';
      ape_author:= ape_titleStmt.getFirstChildElementNS(xmlns_ape,'author');
      while assigned(ape_author) do begin
        xml_lang:= ape_author.getAttributeNodeNS(xmlns_xml,'lang');
        if assigned(xml_lang)
          then currentLanguage:= xml_lang.value
          else currentLanguage:= '';
        if (authorLanguage <> currentLanguage) and  // exclude duplicates
           ( (authorLanguage = 'x-') or
             (currentLanguage = specifiedLanguage) or
             ((currentLanguage = '') and (authorLanguage <> specifiedLanguage)) )
          then begin
          authorLanguage:= currentLanguage;
          newAuthor:= ape_author.textContent;
          if (authorLanguage = specifiedLanguage) then break;
        end;
        ape_author:= ape_author.getNextSiblingElementNS(xmlns_ape,'author');
      end; {while ...}

    end; {if ...}
  end; {if assigned(ape_titleStmt) ...}

  // xxx string vs. wideString
  TitleEdit.Text:= newTitle;
  LabelEdit.Text:= newLabel;
  AuthorEdit.Text:= newAuthor;
end;

end.
