unit ItemEditDlg;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with 
// the License. You may obtain a copy of the License at 
// "http://www.mozilla.org/MPL/"
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is "ItemEditDlg.pas".
// 
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg, 
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
// 
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which 
// case the provisions of the GPL are applicable instead of those above. 
// If you wish to allow use of your version of this file only under the 
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  XDOM_2_4, UnicodeUtils, LanguageAndTextVSTFrame, ApeUtils,
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, ApeUrnListFrame;

type
  TItemEditDialog = class(TForm)
    Panel1: TPanel;
    OKBtn: TButton;
    CancelBtn: TButton;
    DomImplementation1: TDomImplementation;
    Panel2: TPanel;
    CaptionLEdit: TLabeledEdit;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    NoteLTEditor: TApeLanguageAndTextEditor;
    ResourceMapUrnList: TApeUrnListFrame;
    procedure FormCreate(Sender: TObject);
    procedure CaptionLEditChange(Sender: TObject);
    procedure Panel1Resize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FDisplayFontName: TFontName;
    function GetCaptionElement: TdomElement;
    function GetCommentElement: TdomElement;
    function GetItemElement: TdomElement;
    function GetItemDocument: TdomDocument;
    function GetResourceMapElement: TdomElement;
  protected
    function DoStrToWideStrConversion(S: string): wideString; virtual;
    function DoWideStrToStrConversion(S: wideString): string; virtual;
    procedure SetDisplayFontName(const Value: TFontName); virtual;
    procedure UpdateDisplay; virtual;
    procedure UpdateCaption; virtual;
    procedure UpdateNote; virtual;
    procedure UpdateResourceMap; virtual;
    property CaptionElement: TdomElement read GetCaptionElement;
    property CommentElement: TdomElement read GetCommentElement;
    property ItemDocument: TdomDocument read GetItemDocument;
    property ResourceMapElement: TdomElement read GetResourceMapElement;
  public
    procedure InitializeItem(const element: TdomElement); virtual;
    property DisplayFontName: TFontName read FDisplayFontName write SetDisplayFontName;
    property ItemElement: TdomElement read GetItemElement;
  end;

var
  ItemEditDialog: TItemEditDialog;

implementation

{$R *.dfm}

{ TNoteEditDialog }

procedure TItemEditDialog.Panel1Resize(Sender: TObject);
begin
  OKBtn.Left:= (Panel1.Width div 2) - OKBtn.Width - 2;
  CancelBtn.Left:= (Panel1.Width div 2) + 2;
end;

function TItemEditDialog.GetCaptionElement: TdomElement;
var
  itemEl: TdomElement;
begin
  itemEl:= ItemElement;  // Avoid calling GetItemElement() twice
  if assigned(itemEl)
    then result:= itemEl.getFirstChildElementNS(xmlns_ape,'caption')
    else result:= nil;
end;

function TItemEditDialog.GetCommentElement: TdomElement;
var
  itemEl: TdomElement;
begin
  itemEl:= ItemElement;  // Avoid calling GetItemElement() twice
  if assigned(itemEl)
    then result:= itemEl.getFirstChildElementNS(xmlns_ape,'comment')
    else result:= nil;
end;

function TItemEditDialog.GetItemElement: TdomElement;
var
  itemDoc: TdomDocument;
begin
  itemDoc:= ItemDocument;  // Avoid calling GetItemDocument() twice
  if assigned(itemDoc)
    then result:= itemDoc.documentElement
    else result:= nil;
end;

function TItemEditDialog.GetItemDocument: TdomDocument;
begin
  result:= nil;
  with DomImplementation1.documents do
    if length > 0 then result:= item(0) as TdomDocument;
end;

function TItemEditDialog.GetResourceMapElement: TdomElement;
var
  itemEl: TdomElement;
begin
  itemEl:= ItemElement;  // Avoid calling GetItemElement() twice
  if assigned(itemEl)
    then result:= itemEl.getFirstChildElementNS(xmlns_ape,'resourceMap')
    else result:= nil;
end;

procedure TItemEditDialog.InitializeItem(const element: TdomElement);
var
  elementCopy,ape_caption,ape_comment,ape_resourceMap: TdomNode;
begin
  // Make local copy of subtree
  ItemDocument.clear;
  if assigned(element) then begin
    // Do only copy 'ape:caption', 'ape:comment' and 'ape:resourceMap'
    with element do begin
      ape_caption:= getFirstChildElementNS(xmlns_ape,'caption');
      ape_comment:= getFirstChildElementNS(xmlns_ape,'comment');
      ape_resourceMap:= getFirstChildElementNS(xmlns_ape,'resourceMap');
    end;
    with ItemDocument do begin
      elementCopy:= appendChild(importNode(element,false));
      with elementCopy do begin
        appendChild(importNode(ape_caption,true));
        appendChild(importNode(ape_comment,true));
        appendChild(importNode(ape_resourceMap,true));
      end;
    end;
  end;

  UpdateDisplay;
end;

function TItemEditDialog.DoStrToWideStrConversion(S: string): wideString;
begin
  result:= strToPcdata(trim(S),GetACPEncodingType);
end;

function TItemEditDialog.DoWideStrToStrConversion(S: wideString): string;
begin
  result:= trim(pcdataToStr(S,GetACPEncodingType));
end;

procedure TItemEditDialog.UpdateDisplay;
begin
  UpdateCaption;
  UpdateNote;
  UpdateResourceMap;
end;

procedure TItemEditDialog.FormCreate(Sender: TObject);
begin
  DomImplementation1.createDocumentNS(xmlns_ape,'ape:item',nil);
end;

procedure TItemEditDialog.UpdateCaption;
var
  ape_caption: TdomElement;
begin
  ape_caption:= CaptionElement; // Avoid calling GetCaptionElement() twice
  if assigned(ape_caption)
    then CaptionLEdit.Text:= DoWideStrToStrConversion(ape_caption.textContent)
    else CaptionLEdit.Text:= '';
end;

procedure TItemEditDialog.UpdateNote;
begin
  NoteLTEditor.InitializeDisplay(CommentElement,
                                 xmlns_ape,'ape:p',
                                 '','','');
end;

procedure TItemEditDialog.UpdateResourceMap;
var
  ape_resource: TdomElement;
begin
  with ResourceMapUrnList.Lines do begin
    BeginUpdate;
    Clear;
    try
      if assigned(ResourceMapElement) then begin
        ape_resource:= ResourceMapElement.getFirstChildElementNS(xmlns_ape,'resource');
        while assigned(ape_resource) do begin
          add(ape_resource.getAttributeNSNormalizedValue(xmlns_xlink,'href'));
          ape_resource:= ape_resource.getNextSiblingElementNS(xmlns_ape,'resource');
        end;
      end;
    finally
      EndUpdate;
    end;
  end;
end;

procedure TItemEditDialog.CaptionLEditChange(Sender: TObject);
var
  ape_caption: TdomElement;
begin
  ape_caption:= CaptionElement; // Avoid calling GetCaptionElement() twice
  if assigned(ape_caption) then begin
    with ape_caption do begin
      clear;
      appendChild(ownerDocument.createTextNode(DoStrToWideStrConversion((Sender as TLabeledEdit).Text)));
    end;
  end; {if assigned(ape_caption) ...}
end;

procedure TItemEditDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
  i: integer;
  ape_resource: TdomElement;
begin
  if assigned(ResourceMapElement) then begin
    ResourceMapElement.clear;
    with ResourceMapUrnList.Lines do begin
      for i:= 0 to pred(count) do begin
        ape_resource:= ResourceMapElement.ownerDocument.createElementNS(xmlns_ape,'ape:resource');
        ape_resource.setAttributeNS(xmlns_xlink,'xlink:href',wideStrings[i]);
        ResourceMapElement.appendChild(ape_resource);
      end;
    end;
  end;
end;

procedure TItemEditDialog.SetDisplayFontName(const Value: TFontName);
begin
  if FDisplayFontName <> Value then begin
    CaptionLEdit.Font.Name:= Value;
    NoteLTEditor.Font.Name:= Value;
    ResourceMapUrnList.Font.Name:= Value;
  end;
end;

end.
